from hackerforms.widgets.TextareaInput import TextareaInput
from hackerforms.widgets.ListInput import ListInput
from hackerforms.widgets.VideoInput import VideoInput
from hackerforms.widgets.TagInput import TagInput
from hackerforms.widgets.TimeInput import TimeInput
from hackerforms.widgets.NumberSliderInput import NumberSliderInput
from hackerforms.widgets.CheckboxInput import CheckboxInput
from hackerforms.widgets.CnpjInput import CnpjInput
from hackerforms.widgets.CpfInput import CpfInput
from hackerforms.widgets.TextInput import TextInput
from hackerforms.widgets.PhoneInput import PhoneInput
from hackerforms.widgets.ImageInput import ImageInput
from hackerforms.widgets.ToggleInput import ToggleInput
from hackerforms.widgets.ChecklistInput import ChecklistInput
from hackerforms.widgets.EmailInput import EmailInput
from hackerforms.widgets.MultipleChoiceInput import MultipleChoiceInput
from hackerforms.widgets.CodeInput import CodeInput
from hackerforms.widgets.DateInput import DateInput
from hackerforms.widgets.PasswordInput import PasswordInput
from hackerforms.widgets.NumberInput import NumberInput
from hackerforms.widgets.CardsInput import CardsInput
from hackerforms.widgets.AnswerSheetInput import AnswerSheetInput
from hackerforms.widgets.NpsInput import NpsInput
from hackerforms.widgets.CurrencyInput import CurrencyInput
from hackerforms.widgets.FileInput import FileInput
from hackerforms.widgets.RatingInput import RatingInput
from hackerforms.widgets.ClickInput import ClickInput
from hackerforms.widgets.DropdownInput import DropdownInput
from hackerforms.widgets.PandasRowSelectionInput import PandasRowSelectionInput
